<?php
namespace nulib\db\sqlite;

use nulib\db\_private\_base;
use nulib\db\_private\_create;
use nulib\db\_private\_delete;
use nulib\db\_private\_generic;
use nulib\db\_private\_insert;
use nulib\db\_private\_select;
use nulib\db\_private\_update;
use nulib\db\_private\Tbindings;
use nulib\output\msg;
use nulib\ValueException;
use SQLite3;
use SQLite3Stmt;

class query extends _base {
  use Tbindings;

  protected static function verifix(&$sql, ?array &$bindings=null, ?array &$meta=null): void {
    if (is_array($sql)) {
      $prefix = $sql[0] ?? null;
      if ($prefix === null) {
        throw new ValueException("requête invalide");
      } elseif (_create::isa($prefix)) {
        $sql = _create::parse($sql, $bindings);
      } elseif (_select::isa($prefix)) {
        $sql = _select::parse($sql, $bindings);
      } elseif (_insert::isa($prefix)) {
        $sql = _insert::parse($sql, $bindings);
      } elseif (_update::isa($prefix)) {
        $sql = _update::parse($sql, $bindings);
      } elseif (_delete::isa($prefix)) {
        $sql = _delete::parse($sql, $bindings);
      } elseif (_generic::isa($prefix)) {
        $sql = _generic::parse($sql, $bindings);
      } else {
        throw SqliteException::wrap(ValueException::invalid_kind($sql, "query"));
      }
    } elseif (!is_string($sql)) {
      $sql = strval($sql);
    }
  }

  const DEBUG_QUERIES = false;

  function useStmt(SQLite3 $db, ?SQLite3Stmt &$stmt=null, ?string &$sql=null): bool {
    if (static::DEBUG_QUERIES) msg::info($this->sql); #XXX
    if ($this->bindings !== null) {
      /** @var SQLite3Stmt $stmt */
      $stmt = SqliteException::check($db, $db->prepare($this->sql));
      $close = true;
      try {
        foreach ($this->bindings as $param => $value) {
          $this->verifixBindings($value);
          SqliteException::check($db, $stmt->bindValue($param, $value));
        }
        $close = false;
        return true;
      } finally {
        if ($close) $stmt->close();
      }
    } else {
      $sql = $this->sql;
      return false;
    }
  }
}
